package sun.security.jgss;

import org.ietf.jgss.MessageProp;
import java.util.LinkedList;

public class TokenTracker
{
    static final int MAX_INTERVALS = 5;
    private int initNumber;
    private int windowStart;
    private int expectedNumber;
    private int windowStartIndex;
    private LinkedList<TokenTracker.Entry> list;
    
    public TokenTracker(int expectedNumber) {
        this.windowStartIndex = 0;
        this.list = new LinkedList();
        this.initNumber = expectedNumber;
        this.windowStart = expectedNumber;
        this.expectedNumber = expectedNumber;
        this.list.add(new TokenTracker.Entry(expectedNumber - 1));
    }
    
    private int getIntervalIndex(int n) {
        int index;
        for (index = this.list.size() - 1; index >= 0 && ((TokenTracker.Entry)this.list.get(index)).compareTo(n) > 0; --index) {}
        return index;
    }
    
    public final synchronized void getProps(int n, MessageProp messageProp) {
        boolean gap = false;
        boolean old = false;
        boolean unseq = false;
        boolean duplicate = false;
        int intervalIndex = this.getIntervalIndex(n);
        TokenTracker.Entry entry = null;
        if (intervalIndex != -1) {
            entry = (TokenTracker.Entry)this.list.get(intervalIndex);
        }
        if (n == this.expectedNumber) {
            ++this.expectedNumber;
        }
        else if (entry != null && entry.contains(n)) {
            duplicate = true;
        }
        else if (this.expectedNumber >= this.initNumber) {
            if (n > this.expectedNumber) {
                gap = true;
            }
            else if (n >= this.windowStart) {
                unseq = true;
            }
            else if (n >= this.initNumber) {
                old = true;
            }
            else {
                gap = true;
            }
        }
        else if (n > this.expectedNumber) {
            if (n < this.initNumber) {
                gap = true;
            }
            else if (this.windowStart >= this.initNumber) {
                if (n >= this.windowStart) {
                    unseq = true;
                }
                else {
                    old = true;
                }
            }
            else {
                old = true;
            }
        }
        else if (this.windowStart > this.expectedNumber) {
            unseq = true;
        }
        else if (n < this.windowStart) {
            old = true;
        }
        else {
            unseq = true;
        }
        if (duplicate == 0 && old == 0) {
            this.add(n, intervalIndex);
        }
        if (!(gap == 0)) {
            this.expectedNumber = n + 1;
        }
        messageProp.setSupplementaryStates((boolean)duplicate, (boolean)old, (boolean)unseq, (boolean)gap, 0, null);
    }
    
    private void add(int windowStart, int n) {
        TokenTracker.Entry entry = null;
        boolean n2 = false;
        boolean n3 = false;
        if (n != -1) {
            entry = (TokenTracker.Entry)this.list.get(n);
            if (windowStart == entry.getEnd() + 1) {
                entry.setEnd(windowStart);
                n2 = true;
            }
        }
        int index = n + 1;
        if (index < this.list.size()) {
            TokenTracker.Entry entry2 = (TokenTracker.Entry)this.list.get(index);
            if (windowStart == entry2.getStart() - 1) {
                if (n2 == 0) {
                    entry2.setStart(windowStart);
                }
                else {
                    entry2.setStart(entry.getStart());
                    this.list.remove(n);
                    if (this.windowStartIndex > n) {
                        --this.windowStartIndex;
                    }
                }
                n3 = true;
            }
        }
        if (n3 || n2) {
            return;
        }
        Object element;
        if (this.list.size() < 5) {
            element = new TokenTracker.Entry(windowStart);
            if (n < this.windowStartIndex) {
                ++this.windowStartIndex;
            }
        }
        else {
            int windowStartIndex = this.windowStartIndex;
            if (this.windowStartIndex == this.list.size() - 1) {
                this.windowStartIndex = 0;
            }
            element = (TokenTracker.Entry)this.list.remove(windowStartIndex);
            this.windowStart = ((TokenTracker.Entry)this.list.get(this.windowStartIndex)).getStart();
            element.setStart(windowStart);
            element.setEnd(windowStart);
            if (n >= windowStartIndex) {
                --n;
            }
            else if (windowStartIndex != this.windowStartIndex) {
                if (n == -1) {
                    this.windowStart = windowStart;
                }
            }
            else {
                ++this.windowStartIndex;
            }
        }
        this.list.add(n + 1, element);
        return;
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer("TokenTracker: ");
        sb.append(" initNumber=").append(this.initNumber);
        sb.append(" windowStart=").append(this.windowStart);
        sb.append(" expectedNumber=").append(this.expectedNumber);
        sb.append(" windowStartIndex=").append(this.windowStartIndex);
        sb.append("\n\tIntervals are: {");
        for (int i = 0; i < this.list.size(); ++i) {
            if (i != 0) {
                sb.append(", ");
            }
            sb.append(((TokenTracker.Entry)this.list.get(i)).toString());
        }
        sb.append('}');
        return sb.toString();
    }
    
    class Entry
    {
        private int start;
        private int end;
        
        Entry(int n) {
            this.start = n;
            this.end = n;
        }
        
        final int compareTo(int n) {
            if (this.start > n) {
                return 1;
            }
            if (this.end < n) {
                return -1;
            }
            return 0;
        }
        
        final boolean contains(int n) {
            return n >= this.start && n <= this.end;
        }
        
        final void append(int end) {
            if (end == this.end + 1) {
                this.end = end;
            }
        }
        
        final void setInterval(int start, int end) {
            this.start = start;
            this.end = end;
        }
        
        final void setEnd(int end) {
            this.end = end;
        }
        
        final void setStart(int start) {
            this.start = start;
        }
        
        final int getStart() {
            return this.start;
        }
        
        final int getEnd() {
            return this.end;
        }
        
        @Override
        public String toString() {
            return "[" + this.start + ", " + this.end + "]";
        }
    }
}
